Port equ 18h
tempo equ 180

dseg
SoundProc_: public SoundProc_
    defw 0

; void InitSound();
cseg
InitSound_: public InitSound_
    push hl
        ld hl,0
        ld (SoundProc_),hl
    pop hl
ret


; void CallSound();
cseg
CallSound:
jp (hl)
CallSound_: public CallSound_
    push af | push hl
        ld hl,(SoundProc_)
        ld a,l
        or h
        if nz
            call CallSound
            ld hl,0
            ld (SoundProc_),hl
        endif
    pop hl | pop af
ret


Pulse:
; in:
;   bc = cycle
    in a,(Port) 
    or 0c0h
    out (Port), a
    push bc
        do
            dec bc
            ld a,c
            or b
        while nz | wend
    pop bc
    in a,(Port)
    and not 0c0h
    out (Port), a
    push bc
        do
            dec bc
            ld a,c
            or b
        while nz | wend
    pop bc
ret 


Tone:
; in:
;   bc = cycle
;   hl = length
    push de
        ld e,c | ld d,b
        srl d | rr e
        do
            call Pulse
            or a
            sbc hl,de
        while nc | wend
    pop de
ret 


Dummy:
; in:
;   bc = cycle
    push bc
        do
            dec bc
            ld a,c
            or b
        while nz | wend
    pop bc
    push bc
        do
            dec bc
            ld a,c
            or b
        while nz | wend
    pop bc
ret 

Rest:
; in:
;   b = cycle
;   hl = length
    push de
        ld e,c | ld d,b
        srl d | rr e
        do
            call Dummy
            or a
            sbc hl,de
        while nc | wend
    pop de
ret 


; void Sound_Fire();
cseg
Sound_Fire_: public Sound_Fire_
    push af | push hl | push de | push bc
        ld bc, 100*2
        do
            Sound_Fire_.Loop:
            call Pulse
            ld l,c
            ld h,b
            ld de,5*2
            add hl,de
            ld c,l
            ld b,h
            ld a,b
            cp high 300*2
            jr nz,Sound_Fire_.Loop
            ld a,c
            cp low 300*2
        while nz | wend
    pop bc | pop de | pop hl | pop af
ret


; void Sound_Hit();
cseg
Sound_Hit:
    push af | push hl | push de | push bc
        ld bc, 500*2
        do
            Sound_Hit_.Loop:
            call Pulse
            ld l,c
            ld h,b
            ld de,-5
            add hl,de
            ld c,l
            ld b,h
            ld a,b
            cp high 300*2
            jr nz,Sound_Hit_.Loop
            ld a,c
            cp low 300*2
        while nz | wend
    pop bc | pop de | pop hl | pop af
ret 
Sound_Hit_:  public Sound_Hit_
    push hl
        ld hl,Sound_Hit
        ld (SoundProc_),hl
    pop hl
ret



; void Sound_Loose();
cseg
Sound_Loose:
    push af | push bc
        ld bc, 150*2
        do
            Sound_Miss_.Loop:
            call Pulse
            inc bc
            inc bc
            ld a,b
            cp high 300*2 
            jr nz,Sound_Miss_.Loop
            ld a,c
            cp low 300*2
        while nz | wend
    pop bc | pop af
ret 
Sound_Loose_:  public Sound_Loose_
    push hl
        ld hl,Sound_Loose
        ld (SoundProc_),hl
    pop hl
ret


MakeLangth:
    ld h,a
    ld l,0
    add hl,hl ;*2
    ; ld d,h
    ; ld e,l
    add hl,hl ;*4
    add hl,hl ;*8
    add hl,hl ;*16
ret

Melody: 
; in
;   hl: address
    push af
    push de
    push bc
        do
            ld c,(hl) | inc hl
            ld b,(hl) | inc hl
            ld a,c
            or a
        while nz
            if nz
                ld a,b
                or a
                if nz
                    push hl
                        ld a,c
                        dec b
                        ld l,b
                        ld h,0
                        add hl,hl
                        ld de,cycles
                        add hl,de
                        ld c,(hl)
                        inc hl
                        ld b,(hl)
                    pop hl
                    push hl
                        call MakeLangth
                        call Tone
                    pop hl
                else
                    push hl
                        ld a,c
                        ld bc,100*2
                        call MakeLangth
                        call Rest
                    pop hl
                endif
            endif
        wend
    pop bc
    pop de
    pop af
ret


; void Sound_Up();
cseg
up_notes:
    defb 1,C4, 1, C4S, 1,D4, 1,F4, 1,A4, 1,C5, 0
Sound_Up:
    ld hl,up_notes
    call Melody
ret
Sound_Up_:  public Sound_Up_
    push hl
        ld hl,Sound_Up
        ld (SoundProc_),hl
    pop hl
ret


; void Sound_Start();
cseg
start_notes: 
    defb 3,A4, 3,G4, 3,A4, 6,C5, 3,E5, 12,D5,  3,0, 5,C5,1,0, 3,C5, 6,D5
    defb 0
Sound_Start_:
    public Sound_Start_
    push hl
        ld hl,start_notes
        call Melody
    pop hl
ret


; void Sound_Clear();
cseg
clear_notes: 
    defb 3,E4, 3,G4, 3,C5, 3,A4, 3,0, 3,A4, 3,0, 3,G4,  3,0, 3,G4, 3,0, 3,G4, 3,0, 3,G4, 6,A4
    defb 0
Sound_Clear_:
    public Sound_Clear_
    push hl
        ld hl,clear_notes
        call Melody
    pop hl
ret


; void Sound_GameOver();
cseg
over_notes: 
    defb 2,E4,1,0, 3,E4, 3,G4, 6,E4, 3,C5, 3,B4, 3,A4,  3,G4, 3,A4, 3,0, 6,A4, 3,G4, 6,A4
    defb 0
Sound_GameOver_:
    public Sound_GameOver_
    push hl
        ld hl,over_notes
        call Melody
    pop hl
ret



cycles:
defw 588   ;C4
defw 555   ;C#4
defw 523   ;D4
defw 494   ;D#4
defw 466   ;E4
defw 440   ;F
defw 415   ;F#4
defw 392   ;G4
defw 370   ;G#4
defw 349   ;A4
defw 330   ;A#4
defw 311   ;B4
defw 294   ;C5
defw 277   ;C#5
defw 261   ;D5
defw 247   ;D#5
defw 233   ;E5
defw 220   ;F5


C4 equ 1
C4S equ 2
D4 equ 3
E4 equ 5
F4 equ 6
G4 equ 8
A4 equ 10
B4 equ 12
C5 equ 13
D5 equ 15
E5 equ 17
F5 equ 18
