#include "Status.h"
#include "Vram.h"
#include "Sprite.h"
#include "Print.h"
#include "Chars.h"
#include "VVram.h"
#include "Main.h"
#include "Rader.h"

constexpr byte ScoreLeft = 24 + 1;
constexpr byte Char_Remain = Char_Fighter;

static ptr<byte> PrintS(ptr<byte> pVram, ptr<byte> p)
{
    byte c;
    while ((c = *p) != 0) {
        pVram = PrintC(pVram, c);
        ++p;
    }
    return pVram;
}

void PrintStatus() 
{
    PrintS(Vram + VramRowSize * 0 + ScoreLeft * VramStep, "SCORE");
    PrintS(Vram + VramRowSize * 1 + ScoreLeft * VramStep, "STAGE");
    PrintByteNumber2(Vram + VramRowSize * 1 + (ScoreLeft + 9) * VramStep, CurrentStage + 1);
    PrintScore();
    PrintRemain();
    PrintRader();
}

void PrintScore()
{
    ptr<byte> pVram;
    
    pVram = PrintNumber5(Vram + VramRowSize * 0 + (ScoreLeft + 5) * VramStep, Score);
    PrintC(pVram, '0');
}

void PrintRemain()
{
    ptr<byte> pVram;
    pVram = Vram + VramRowSize * 5 + ScoreLeft * VramStep;
    if (RemainCount > 1) {
        byte i;
        i = RemainCount - 1;
        if (i > 2) {
            pVram = Put2C(pVram, Char_Remain);
            pVram = Erase2(pVram);
            pVram += VramRowSize - 2;
            pVram = PrintC(pVram, i + 0x30);
            pVram += 1 - VramRowSize;
        }
        else {
            do {
                pVram = Put2C(pVram, Char_Remain);
                --i;
            } while (i > 0);
        }
    }
    if (RemainCount < 3) {
        pVram = Erase2(pVram);
    }
}

void PrintGameOver()
{
    PrintS(Vram + VramRowSize * 3 + (VVramWidth - 9) / 2 * VramStep, "GAME OVER");
    PresendVram();
}

void Title()
{
    ClearScreen(); 
    HideAllSprites();
    PrintStatus(); 
    {
        static const byte[] TitleBytes = {
            0x0f, 0x0d, 0x02, 0x0f, 
            0x0f, 0x0e, 0x01, 0x0f, 
            0x0f, 0x0c, 0x03, 0x0f, 
            0x05, 0x04, 0x01, 0x04, 
            0x0c, 0x03, 0x0f, 0x0d, 
            0x0c, 0x03, 0x0f, 0x0e, 
            0x0c, 0x03, 0x0f, 0x05, 
            0x05, 0x00, 0x05, 0x00, 
            0x02, 0x0d, 0x07, 0x0c, 
            0x03, 0x0c, 0x03, 0x0c, 
            0x00, 0x0c, 0x03, 0x0c, 
            0x00, 0x04, 0x01, 0x00, 
            0x03, 0x0f, 0x08, 0x07, 
            0x03, 0x0f, 0x04, 0x0b, 
            0x03, 0x0f, 0x08, 0x02, 
            0x05, 0x01, 0x00, 0x05, 
            0x0b, 0x00, 0x00, 0x00, 
            0x02, 0x00, 0x00, 0x00, 
            0x0f, 0x00, 0x00, 0x00, 
            0x01, 0x00, 0x00, 0x00, 
        };
        constexpr byte LogoLength = 5;
        ptr<byte> p;
        ptr<byte> pVram;
        pVram = Vram + VramRowSize * 0 + (VVramWidth - LogoLength * 4) * VramStep / 2;
        p = TitleBytes;
        repeat (LogoLength) {
            byte shift;
            shift = 3;
            repeat (4) {
                repeat (4) {
                    pVram = Put(pVram, *p, shift);
                    ++p;
                }
                pVram -= 4 * CharWidth;
                shift += 3;
                if (shift >= 8) {
                    shift -= 8;
                    pVram += VramRowSize;
                }
            }
            pVram += 4 * CharWidth - VramRowSize;
        }
        PrintS(Vram + VramRowSize * 2 + (VVramWidth - 8) * CharWidth, "MINI");
    }
    PrintS(Vram + VramRowSize * 3 + 1 * VramStep, "PUSH SPACE KEY");
    PrintS(Vram + VramRowSize * 4 + 1 * VramStep, "OR KANA KEY");
    PrintS(Vram + VramRowSize * 5 + 4 * VramStep, "INUFUTO 2022");
    PresendVram();
}
